from uuid import uuid5, NAMESPACE_URL

from sqlalchemy import (
    Column,
    Integer,
    ForeignKey,
    String,
)
from sqlalchemy_utils import generic_repr
from sqlalchemy.dialects.postgresql import JSONB

from vdl_tools.shared_tools.database_cache.models.base import BaseMixin


PROMPT_RESPONSE_NAMESPACE_TEXT = "PROMPT_RESPONSE_NAMESPACE_TEXT"


def make_uuid(name, namespace_text=PROMPT_RESPONSE_NAMESPACE_TEXT):
    namespace = uuid5(NAMESPACE_URL, namespace_text)
    return str(uuid5(namespace, name=name))


@generic_repr
class Prompt(BaseMixin):
    """Table to hold scraped prompts"""
    __tablename__ = 'prompt'

    id = Column(String, primary_key=True)
    name = Column(String, primary_key=False, nullable=True)
    description = Column(String, nullable=True)
    prompt_str = Column(String, nullable=False)

    def __init__(self, **kwargs):
        if 'id' not in kwargs:
            kwargs['id'] = make_uuid(
                name=kwargs["prompt_str"],
                namespace_text=PROMPT_RESPONSE_NAMESPACE_TEXT
            )
        else:
            assert kwargs['id'] == make_uuid(
                name=kwargs["prompt_str"],
                namespace_text=PROMPT_RESPONSE_NAMESPACE_TEXT
            )
        super().__init__(**kwargs)


class PromptResponse(BaseMixin):
    """Table to hold the responses for prompts"""
    __tablename__ = 'prompt_response'

    prompt_id = Column(String, ForeignKey('prompt.id', onupdate="CASCADE", ondelete="CASCADE"), primary_key=True)
    given_id = Column(String, primary_key=True)
    text_id = Column(String)
    input_text = Column(String, nullable=False)
    response_full = Column(JSONB, nullable=False)
    response_text = Column(String, nullable=True)
    num_errors = Column(Integer, nullable=True)

    def __init__(self, **kwargs):
        if 'text_id' not in kwargs:
            text_id = make_uuid(
                name=kwargs["input_text"],
                namespace_text=PROMPT_RESPONSE_NAMESPACE_TEXT
            )
            kwargs['text_id'] = text_id
        else:
            assert kwargs['text_id'] == make_uuid(
                name=kwargs["input_text"],
                namespace_text=PROMPT_RESPONSE_NAMESPACE_TEXT
            )
        super().__init__(**kwargs)
